﻿
namespace ListPatternMatching {
	internal class Program {

		static void PrintAll(int[] values) {
            Console.Write("Array int[] values: ");
            for (int i = 0; i < values.Length; i++) {
				Console.Write(values[i]);
                Console.Write(' ');
            }
            Console.WriteLine();
        }

		static void Main(string[] args) {

			int[] a = { 1, 2, 3, 4, 5, 6, 7 };

			{
				if (a is [< 2, > 1, .. int[] restArray, int x]) {
					Console.WriteLine($"x == {x}");

					restArray[0] = 100;

					PrintAll(a);
					PrintAll(restArray);
				}
			}

			{
				if (a is [< 2, > 1, .. var restArray, var x]) {
					Console.WriteLine($"x == {x}");

					restArray[0] = 100;

					PrintAll(a);
					PrintAll(restArray);
				}
			}

			{
				// Error: An expression of type 'int[]' cannot be handled by a pattern of type 'List<int>'.
				// _ = a is [< 2, > 1, .. List<int> restArray, var x];
			}

			Console.WriteLine();

			List<int> list = new List<int>() { 1, 2, 3, 4, 5, 6, 7 };

			{
				if (list is [< 2, > 1, .. List<int> restList, var x]) {
					Console.WriteLine($"x == {x}");

					restList[0] = 100;

					PrintAll(list);
					PrintAll(restList);
				}
			}

			{
				if (list is [< 2, > 1, .. var restList, var x]) {
					Console.WriteLine($"x == {x}");

					restList[0] = 100;

					PrintAll(list);
					PrintAll(restList);
				}
			}

			{
				// Error: An expression of type 'List<int>' cannot be handled by a pattern of type 'int[]'.
				// _ = list is [< 2, > 1, .. int[] restList, var x];
			}

			Console.WriteLine(Environment.NewLine + "-------- Collection initializer 1 - array --------");

            int[] b = [1, 2, 3];
			PrintAll(b);

			Console.WriteLine(Environment.NewLine + "-------- Collection initializer 2 - array --------");

			int[] c = [-1, .. a, -2, .. a, -3];
			PrintAll(c);

			Console.WriteLine(Environment.NewLine + "-------- Collection initializer 3 - List --------");

			List<int> bl = [1, 2, 3];
			PrintAll(bl);

			Console.WriteLine(Environment.NewLine + "-------- Collection initializer 4 - List --------");

			List<int> cl = [-1, .. a, -2, .. a, -3];
			PrintAll(cl);

            // ADVANCED = summer semester spoiler:

            Console.WriteLine();
            Console.WriteLine(">>> ADVANCED >>>");
            {
				if (list is [< 2, > 1, .. IList<int> restList, var x]) {
					Console.WriteLine($"x == {x}");

					restList[0] = 100;

					PrintAll(a);
					PrintAll(restList);
				}
			}

            Console.WriteLine();

            {
				if (list is [< 2, > 1, .. IReadOnlyList<int> restList, var x]) {
					Console.WriteLine($"x == {x}");

					// restList[0] = 100;

					PrintAll(a);
					PrintAll(restList);
				}
			}

            Console.WriteLine();

            {
				if (list is [< 2, > 1, .. IEnumerable<int> restList, var x]) {
					Console.WriteLine($"x == {x}");

					// restList[0] = 100;

					PrintAll(a);
					PrintAll(restList);
				}
			}
		}

		static void PrintAll(List<int> list) {
			Console.Write("List<int> values: ");
			for (int i = 0; i < list.Count; i++) {
				Console.Write(list[i]);
				Console.Write(' ');
			}
			Console.WriteLine();
		}

		static void PrintAll(IEnumerable<int> list) {
			Console.Write("IEnumerable<int> values: ");
			foreach (int item in list) {
				Console.Write(item);
				Console.Write(' ');
			}
			Console.WriteLine();
		}
	}
}
