﻿
namespace NezarkaBookstore {

	class BooksController : IController {
		private ModelStore store;
		private IBooksViewFactory booksViewFactory;
		private IBooksDetailViewFactory booksDetailViewFactory;

		public BooksController(ModelStore store, IBooksViewFactory booksViewFactory, IBooksDetailViewFactory booksDetailViewFactory) {
			this.store = store;
			this.booksViewFactory = booksViewFactory;
			this.booksDetailViewFactory = booksDetailViewFactory;
		}

		public ViewOrError HandleRequest(Customer customer, string? action, int id) {
			switch (action) {
				case null:
					return new ViewOrError(booksViewFactory.CreateView(store));

				case "Detail":
					var book = store.GetBook(id);
					if (book is null) {
						return new ViewOrError(new BookNotFoundError(id));
					} else {
						return new ViewOrError(booksDetailViewFactory.CreateView(book));
					}

				default:
					return new ViewOrError(new InvalidActionError(action));
			}
		}
	}

	record class BookNotFoundError(int BookId) : Error { } 
}