﻿using System;
using System.Collections.Generic;

namespace NezarkaBookstore {

	class Book {
		public required int Id { get; set; }
		public required string Title { get; set; }
		public required string Author { get; set; }
		public decimal Price { get; set; }
	}

	class Customer {
		private ShoppingCart? shoppingCart;

		public required int Id { get; set; }
		public required string FirstName { get; set; }
		public required string LastName { get; set; }

		public ShoppingCart ShoppingCart {
			get {
				if (shoppingCart is null) {
					shoppingCart = new ShoppingCart();
				}
				return shoppingCart;
			}
			set {
				shoppingCart = value;
			}
		}
	}

	class ShoppingCartItem {
		public required int BookId { get; set; }
		public int Count { get; set; }
	}

	class ShoppingCart {
		public int CustomerId { get; set; }
		public List<ShoppingCartItem> Items = new List<ShoppingCartItem>();

		public void Add(Book book) {
			var item = Items.Find(i => i.BookId == book.Id);
			if (item is null) {
				item = new ShoppingCartItem { BookId = book.Id, Count = 0 };
				Items.Add(item);
			}
			item.Count++;
		}

		public bool Remove(Book book) {
			var item = Items.Find(i => i.BookId == book.Id);
			if (item is null) {
				return false;
			} else {
				item.Count--;
				if (item.Count == 0) {
					Items.Remove(item);
				}
				return true;
			}
		}
	}

}